/**
 * @page IPI
 *
 * <h3> Interprocessor Interrupts </h3>
 *
 * Our system is able to run on multi processor system. That's why we need 
 * same to communicate in between CPUs. Out platform supports interprocessor
 * interrupts via dorder device. 
 * 
 * We use these interrupts to signalize @link panic @endlink and @link tlb_clear @endlink.
 * To able to distinguish between these too we need some data structures to fill with
 * the cause of IPI and the send it. That was simple when we used it only to call these
 * methods. All we needed was a signle atomic variable indicating which of them should 
 * CPU handle. 
 *
 * Later we added more effective functions like @link tlb_clear_asid @endlink and
 * @link tlb_clear_range @endlink. Now there's need not to pass just a flag indicating
 * the reason to other CPUs we must supply more data like the asid or the range.
 *
 * We didn't want to use single structure synchronized over all CPUs. We came with 
 * solution based on non blocking lists and queue of IPIs. Every CPU would have it's
 * own list of IPI reason and CPU firing the IPI would fill other CPUs' queues. 
 * This would not require any other synchronization and every CPU would be 
 * responsible for it's own queue only. There will be no need for other acknowledgments
 * and synchronization.
 *
 * Unfortunately due to lack of time we didn't extended our system with it.
 *
 * @link kernel/mm/tlb.h TLB implementation @endlink
 *
 * @link kernel/exc/assert.h Kernel debug tools support @endlink
 *
 * @link kernel/drivers/dorder Interprocessor device support @endlink
 */ 
