/** 
 * @page sync
 *
 * <h3> Synchronization </h3>
 *
 * All synchronization primitives are internally implemented by semaphores.
 *
 * Standard @link kernel/sync/semaphore.h semaphore @endlink contains free space 
 * counter (indicating how many threads can low the semaphore without being blocked), 
 * waiting queue (for blocked threads) and spinlock (for ensuring atomicity in 
 * critical sections). Critical sections are also executed with disabled interrupts 
 * so scheduler is not able to plan it out (it causes faster execution). When some 
 * thread tries to lower the semaphore when counter is zero this thread is blocked 
 * on semaphore. When a thread tries to raise the semaphore while other thread is 
 * blocked there, the semaphore counter is not raised but the blocked thread is 
 * planned instead.
 *
 * @link kernel/sync/spinlock.h Spinlock @endlink is implemented by link load and 
 * store conditional instructions provided by MIPS instruction set. It ensures SMP 
 * safety for critical sections (disabling interrupts is enough for one CPU).
 *
 * @link kernel/sync/mutex.h Mutex @endlink is realized by semaphore with counter 
 * initialized to value 1. Mutex locking is realized by semaphore lowering, mutex 
 * unlocking is realized by semaphore raising. There are two special cases which 
 * must be considered. Firstly, mutex has owner while is locked. That owner is 
 * always the thread which lowered the semaphore as the last one. So by remembering 
 * this thread we have the mutex owner. Secondly, as long as mutex is implemented 
 * with standard semaphore, it can be pre-unlocked when the semaphore is repeatedly 
 * raised. This is solved in semaphore down function which allows to set the maximum
 * value to which can be the semaphore raised.
 *
 * @link kernel/sync/rwlock.h RWlock @endlink is implemented by standard mutex and 
 * the semaphore with some modified functions. Mutex is the write lock there (only 
 * one writer is allowed to pass the critical section) and semaphore is read lock. 
 * To allow possible any number of readers to pass the critical section, the semaphore 
 * has a special function which allows to lower it anyway without thread blocking, 
 * therefore the counter in semaphore can reach negative values. When a reader 
 * wants to enter the critical section, he tries to lock and unlock the write lock 
 * to ensure that no writer is in that section and lowers the reader lock (semaphore) to
 * register that reader into the critical section. When a reader leaves the
 * critical section, it raises the reader lock to unregister that reader. When
 * a writer wants to enter the critical section, it locks the write lock (mutex)
 * to not allow any writer nor other reader to enter that section and
 * it also tries to lower the reader lock by normal way (as well as standard
 * semaphore locking) to ensure that the writer is not able to enter the section
 * while some reader is inside. When a writer leaves the critical section, it
 * unlocks the write lock to allow other threads to enter the critical section.
 *
 * @link kernel/sync/cvar.h Conditional variable @endlink is implemented by one 
 * semaphore with enhanced functions. Waiting on condition variable is realized 
 * by lowering semaphore (enhanced semaphore down function is able to unlock the 
 * mutex given by conditional variable), signal is realized by raising semaphore. 
 * Broadcast is implemented by releasing all waiting threads on semaphore and 
 * incrementing the counter once for each released thread.
 */
