/** 
 * @page Syscalls
 *
 * <h3> Syscalls </h3>
 *
 * Each userland thread has one kernel stack to realize syscalls there.
 *
 * When a syscall is called from userland (by syscall instruction), kernel handles
 * it just by switching to kernel stack of that thread. There is potential endless
 * loop on that stack. That loop handles syscalls one by one and when it goes to
 * the end of one cycle of that loop, it switches to the userland stack which can
 * continue from the point where it calls the syscall.
 *
 * In almost every syscall there is neccessary to pass some arguments from the
 * userland into the kernel mode and some results from the kernel mode into the
 * userland. It is possible to pass some 32 bit unsigned integers to the kernel
 * mode and back. It is realized by registers of the userspace context and we use
 * a0, a1, a2 and a3 registers for it. Kernel part of thread can read them and also
 * modify them to return some value to the userland.
 *
 * For bigger data transfers between userland and kernel parts we can use that
 * integers as pointers because both parts share one address space. Kernel part of
 * that thread can access everything in that address space. However, the userland
 * part is not able to access (read/write/modify) memory allocated from kernel
 * mode (unless the kernel part allocates it especially for the userland part).
 *
 * In each thread structure there is a flag that indicate if the thread is in kernel
 * or userland part to let the scheduler know to which stack it shall switch when
 * the thread is planned to run.
 *
 * @link kernel/exc/syscall.h Kernel syscall implementation @endlink
 *
 * @link syscall/syscall.h Syscall interface @endlink
 *
 * @link user/exc/syscall.h User syscall implementation @endlink
 */
