/** 
 * @page vmm
 *
 * <h3> Virtual memory manager </h3>
 *
 * Depending on the current mode (user, supervisor, kernel) is a virtual address space of processor MIPS R4000 divided into 
 * several segments.
 *
 * In user mode is accessible only segment USEG for addresses in the range [0x00000000, 0x800000000), which is mapped to physical
 * memory through the TLB. Access to an address outside USEG throws Address Error Exception.
 *
 * In supervisor mode are defined segments SUSEG in the address range [0x00000000, 0x80000000) and the segment SSEG in address range
 * [0xC0000000, 0xE00000000). Addresses of both segments are mapped to physical memory through the TLB, whereas analogy of SUSEG 
 * segment is USEG segment in user mode. Access to addresses outside SUSEG or SSEG throws Address Error Exception.
 *
 * And finally in kernel mode are defined segments KUSEG in the address range [0x00000000, 0x80000000), KSEG0 in the address range 
 * [0x80000000, 0xA0000000), KSEG1 in the address range [0xA0000000, 0xC0000000), KSSEG/KSEG2 in the address range [0xC0000000, 
 * 0xE0000000) and the KSEG3 in the address range [0xE0000000, 0xFFFFFFFF]. Addresses in segments KUSEG, KSSEG, and KSEG3 are mapped 
 * into physical memory through the TLB, whereas KUSEG segment respectively KSSEG segment is the analogy of USEG segment respectively 
 * SSEG segment in user mode, respectively supervisor mode. Addresses in KSEG0 and KSEG1 segments are mapped to physical memory 
 * directly with standing identical mapping (only subtracting starting address of the segment). It means that virtual addresses in 
 * the segment KSEG0 is mapped to physical addresses in the range [0x00000000, 0x20000000) and in the same way for KSEG1.
 *
 * Because the virtual address processing in individual segments differ, the virtual address space is typically allocated for 
 * contiguous areas (virtual memory area) within these segment. It means, the virtual memory area represents the contiguous block of
 * virtual address space. Depending on in which segment of virtual address space the area is located, then the mapping the range of 
 * virtual addresses into physical memory either fixed (case of KSEG0 and KSEG1 segment) or user-defined (operating system), which 
 * has the possibility to influence the contents of the TLB.
 *
 * The virtual address space is one of the resources shared by all threads one user process (in this context this is definition of 
 * process). For security reasons, each process has its own virtual address space.
 *
 * Virtual memory manager provides virtual blocks allocation and virtual addresses translation. The information about 
 * every virtual block (or virtual page) for given process is stored in a virtual area table (or virtual translation 
 * table). In this context we define the process as a group of threads with the same virtual space (it means the same 
 * virtual translation table and area table). The number of virtual areas in one virtual space is limited by the constant 
 * VM_MAX_AREAS.
 *
 * To decrease memory complexity, the virtual translation table is designed to 3-level translation at the cost of the 
 * more memory accesses (higher execution speed complexity). The first 10 bits from the address is used as index to 
 * master index table to get a pointer to second index table. Then next 10 bits is used as index in the second index 
 * table to get a pointer to the physical address of page (frame in the physical memory context) for the given virtual 
 * address. The remaining 12 bits of the virtual address is joined to this physical address to get the final physical 
 * address.
 *
 * Example: The virtual address 0xdeadbeef in hexadecimal notatin, 11011110 10101101 10111110 11101111 in binary notation. The first
 * 10 bits (1101111010) will be used as index to master index table to get the second index table. In the same way the next 10 bits 
 * (1011011011) will be used with second index table to get the physical address of page (frame). Then the remaining 12 bits
 * (1011101111) will be joined with the physical address of page (frame) to get the final physical address.
 *
 * The userspace protection is provided by syscalls, virtual memory manager only checks the accessability of block asked to allocate
 * (e.g. in user mode cannot be allocated any block in KSEG3 segment). In this context the virtual memory manager is designed a little 
 * generally. In this kind of system we cannot employ the additionally options of its method.
 *
 * @link kernel/mm/vmm.h Virtual memory manager implementation @endlink
 
 * @link kernel/mm/tlb.h TLB implementation @endlink
 *
 * @see vma_map
 * @see vma_unmap
 */
