/**
 * @file hashmap.h
 * Generic hashmap manipulation functions.
 *
 * Originally by Elliot C Back - http://elliottback.com/wp/hashmap-implementation-in-c/
 *
 * Modified by Pete Warden to fix a serious performance problem, support strings as keys and removed thread synchronization 
 * http://petewarden.typepad.com
 *
 * Modified by Michal Drobny to support long integers (~ pointers) as keys. Integration to OS - DrHaUr.
 *
 * @see http://elliottback.com/wp/hashmap-implementation-in-c/
 * @see https://github.com/petewarden/c_hashmap
 */
 
#ifndef __HASHMAP_H__
#define __HASHMAP_H__

/** No such element. */
#define MAP_MISSING -3

/** Hashmap is full. */
#define MAP_FULL -2

/** Out of Memory. */
#define MAP_OMEM -1

/** OK. */
#define MAP_OK 0 	


/**
 * any_t is a pointer.
 * This allows you to put arbitrary structures in the hashmap.
 */
typedef void* any_t;


/**
 * PFany is a pointer to a function.
 * Can take two any_t arguments and return an integer. Returns status code.
 */
typedef int (*PFany)(any_t, any_t);


/**
 * map_t is a pointer to an internally maintained data structure.
 * Clients of this package do not need to know how hashmaps are represented. They see and manipulate only map_t's.
 */
typedef any_t map_t;


/* Commented with implementation. */

unsigned long 	crc32 		(const unsigned long nmb, unsigned int len);
map_t 		hashmap_new 	(void);
int 		hashmap_iterate (map_t in, PFany f, any_t item);
int 		hashmap_put 	(map_t in, unsigned long key, any_t value);
int 		hashmap_get 	(map_t in, unsigned long key, any_t* arg);
int 		hashmap_remove 	(map_t in, unsigned long key);
int 		hashmap_get_one (map_t in, any_t* arg, int remove);
void 		hashmap_free 	(map_t in);
int 		hashmap_length 	(map_t in);


#endif