/**
 * @file list.h
 *
 * Simple doubly linked list implementation.
 *
 * Kalisto
 *
 * Copyright (c) 2001-2010
 *   Department of Distributed and Dependable Systems
 *   Faculty of Mathematics and Physics
 *   Charles University, Czech Republic
 *
 */

#ifndef LIST_H_
#define LIST_H_


#include <include/c.h>


/* Forward declaration */
struct item;


/** A doubly linked list
 *
 */
typedef struct list {
	/** The first item on the list or NULL if empty. */
	struct item *head;
	
	/** The last item on the list or NULL if empty. */
	struct item *tail;
} list_t;


/** An item of a doubly linked list
 *
 * The item should be first in listable structures.
 *
 */
typedef struct item {
	/** The list that we currently belong to. */
	struct list *list;
	
	/** The next item on the list or NULL if first. */
	struct item *prev;
	
	/** The previous item on the list or NULL if last. */
	struct item *next;
} item_t;


/* Externals are commented with implementation. */
extern void list_init (list_t *list);
extern void item_init (item_t *item);
extern void list_append (list_t *list, item_t *item);
extern void list_insert( list_t* list, item_t* item, uint32_t position);
extern void list_remove (list_t *list, item_t *item);
extern item_t *list_rotate (list_t *list);


#endif
