/**
 * @file sync_list.h
 *
 * Implementation of global synchronized single linked list.
 * Synchronization is provided using the mutex synchronization primitive. There is a global mutex, which was locked,
 * when someone operates with the list.
 * @link kernel/sync/mutex.h Kernel mutex implementation @endlink
 */

#ifndef SYNC_LIST_H_
#define SYNC_LIST_H_


/** Structure of an item of linked list. */
typedef struct sync_item 
{
	/** The key. */
	unsigned long key;
	
	/** The data. */
	void* data;
	
	/** The next item in the list. */
	struct sync_item * next;
	
} sync_item_t;


/** Structure of synchronized linked list. */
typedef struct sync_list 
{
	/** The first item in the list. */
	sync_item_t *head;
	/** The last item in the list. */
	sync_item_t *tail;
	
} sync_list_t;


/* Commented with implementation. */

void 		sync_list_init 		(sync_list_t* list);
bool 		sync_list_insert 	(sync_list_t* list, unsigned long key, void* data);
bool 		sync_list_remove 	(sync_list_t* list, unsigned long search_key, void** data_ptr);
bool 		sync_list_find 		(sync_list_t* list, unsigned long search_key, void** data_ptr);
void 		sync_list_dump 		(sync_list_t* list);

#endif
