/**
 * @file dorder.h
 *
 * Dorder device support.
 *
 * Kalisto
 *
 * Copyright (c) 2001-2010
 *   Department of Distributed and Dependable Systems
 *   Faculty of Mathematics and Physics
 *   Charles University, Czech Republic
 *
 */

#ifndef DORDER_H_
#define DORDER_H_


#include <adt/atomic.h>
#include <exc/int.h>
#include <include/c.h>
#include <include/shared.h>


#define DORDER_ADDRESS  (ADDR_IN_KSEG1 (DEVICE_DORDER_ADDR))


/** 
 * Get the ID of the current CPU
 *
 * @return Identification number of current CPU
 *         (range 0 - 31).
 */
static inline uint32_t cpuid( void)
{
	return (*(volatile uint32_t *) DORDER_ADDRESS);
}


/** 
 * Send interprocessor panic interrupt to all the CPUs.
 */
static inline void cpu_send_ipi_panic_all (void)
{
	atomic_set (&ipi_panic, 1);
	* ((volatile unsigned int *) DORDER_ADDRESS) = 0xffffffff;
}


/** 
 * Send interprocessor interrupt to all the CPUs with given cause.
 */
static inline void cpu_send_ipi_cause_all (int32_t cause)
{
	atomic_set (&ipi_cause, cause);
	* ((volatile unsigned int *) DORDER_ADDRESS) = 0xffffffff;
}


/**
 * Deassert interprocessor interrupt for the given CPU.
 *
 * @param id
 *	ID of CPU to deassert interrupt.
 */
static inline void cpu_ack_ipi (uint32_t id)
{
	* ((volatile unsigned int *) (DORDER_ADDRESS + 4)) = (1 << id);
}

#endif
