/**
 * @file dtime.c
 *
 * Dtime device support.
 *
 */

#include <drivers/dtime.h>

#include <drivers/printer.h>


/**
 * Aproximately measures how many ticks passes per second. 
 * As long as we are running on simulator this functions is just for fun.
 *
 * @retun
 *	Tickes per second.
 */
uint32_t ticks_per_second (void)
{
	uint32_t ticks = 0;
	uint32_t clock = time () + 2;

	/* Wait to start on a time change. */ 
	while (clock > time ()); 

	/* Start. */
	write_cp0_count (0);

	clock += 1;
	/* Meassure ticks for one second. */
	while (clock > time ())
	{
		if (read_cp0_count () >= 0xF0000000) 
		{
			/* Either 64 bit registers should be supported on the platform
			 * or algorithm needs to take into account ticks overflows. */ 
			panic ("DTIME ticks overflow");
		}
	}
	/* Actual measure ticks per seconds. */
	ticks = read_cp0_count ();
	printk_not_sync ("ticks per second: %d\n", ticks);

	/* To keep msim nice and deterministic for debugging. */
	ticks = 8000000;	
	printk_not_sync ("setting statically to: %d\n", ticks);

	return ticks;
}
