/**
 * @file timer.h
 *
 * Timer queue and structures implementation.
 *
 */

#ifndef TIMER_H_
#define TIMER_H_

#include <include/c.h>
#include <include/error_codes.h>
#include <proc/thread.h>


/**
 * Structure to represent a timer in the kernel.
 */
struct timer;
typedef struct timer {

	/* timer handler function */
	void (*handler) (struct timer *, void *);
	
	/* timer data to be passed to the handler function */
	void *data;

	/* time to wait before running handler */
	uint32_t usec;

	/* remaining time in usec */
	uint32_t remain_usec;

	/* remaining time in ticks, converted usec per usec by ratio */
	uint32_t remain_ticks;

	/* true if timer is started and in the queue, false if it is not */
	bool enabled;

	bool free_me;

	/* used for system intern timer queue */
	struct timer *prev;
	
	/* used for system intern timer queue */
	struct timer *next;
} timer;


/**
 * Sets hw timer interrupt by specified quantum. 
 *
 * @param quantum
 * 	Quantum of ticks before firing hw interrupt. 
 */
static inline uint32_t timer_setup (uint32_t quantum); 
static inline uint32_t timer_setup (uint32_t quantum) 
{
	uint32_t ticks = read_cp0_count ();
	write_cp0_count (0);
	write_cp0_compare (quantum);
	return ticks;
} 


/* Commented with implementation. */

void		init_timer (void);
void 		init_timer_cpu (void);
int32_t		timer_init (
			struct timer *tmr,
			const uint32_t usec,
			void (*handler) (struct timer *, void *), 
			void *data);
void 		timer_start (struct timer *tmr);
void		timer_destroy (struct timer *tmr); 
int32_t		timer_pending (struct timer *tmr);
void		timer_ticks (uint32_t ticks);
bool		timer_is_ready (void);
struct timer* 	timer_ready (void);
void 		timer_dump (void);


#endif
