/**
 * @file memtest.h
 *
 * Detecting memory on system start by writting patterns to it and reading them.
 * Because there can be more memory modules in system, we do not assume one solid
 * memory block, there can be holes in the address space. Each solid block is 
 * count as one hardware memory module. We assume that there is no more than 
 * PHYSICAL_MEMORY_MODULES_LIMIT in the system. We left a free space in memory for
 * kernel binary and userland thread binary. This memory is treat as not available,
 * so it can not be allocated or freed (as like as address ranges where are no 
 * memory modules).
 *
 */


#ifndef MEMTEST_H_
#define MEMTEST_H_


#include <include/c.h>


#define PHYSICAL_MEMORY_LIMIT_FOR_4K_FRAMES (1 << 20)
#define PHYSICAL_MEMORY_MODULES_LIMIT       32

#define USER_BIN_START 0x08000000
#define USER_BIN_LEN   (1 << 22)

#define KERNEL_HEAP_SIZE 1048576

extern uint8_t  available_memory_map [];
extern uint32_t first_frames_of_memory_moduls [];
extern uint32_t frames_count_in_memory_moduls [];
extern uint8_t  _kernel_end;


/* find out how much memory is in system */
void memory_test (void);


#endif