/**
 * @file physmem.h
 *
 * Physical memory management.
 *
 * Pyhsical memory is managed by a bitmap. There is allowed to have more than one
 * physical memory module (more physical devices). Memory map as well as modules
 * starts and ends addresses are initialized in memtest.h. Here is just allocating
 * and freeing frames, all of it is very easy to understand. Free blocks are found
 * by first-fit strategy.
 *
 */


#ifndef PHYSMEM_H_
#define PHYSMEM_H_


#include <mm/memtest.h>


#define PHYSMEM_PAGE_SIZE 4096
#define FRAME_SIZE        PHYSMEM_PAGE_SIZE

#define VF_VA_AUTO  (1 << 0)
#define VF_VA_USER  (1 << 1)
#define VF_AT_KSEG0 (1 << 2)
#define VF_AT_KSEG1 (1 << 3)
#define VF_AT_KUSEG (1 << 4)
#define VF_AT_KSSEG (1 << 5)
#define VF_AT_KSEG3 (1 << 6)
#define VM_SAFE_RANGE (1 << 7)


typedef uint32_t vm_flags_t;


/* initializes physical memory mapping management */
void physical_memory_mapping_init (void);

/* allocates physical memory in one block */
int frame_alloc (void ** paddr, const size_t cnt, const vm_flags_t flags);

/* frees physical memory allocated by frame_alloc */
int frame_free (const void * paddr, const size_t cnt);


#endif