/**
 * @file tlb.h
 *
 * Address translation handling.
 *
 */

#ifndef TLB_H_
#define TLB_H_


#include <include/c.h>
#include <proc/thread.h>


/** Number of ASIDs. 
 */ 
#define ASID_LENGTH 256

/** ASID map of current usage. 
 * - 0 if ASID is not used
 * - 1 if it is 
 */
int32_t asid_used [MAX_CPU] [ASID_LENGTH];

/** Pointers of threads mapped by ASIDs. 
 */
thread_t asid_thread [MAX_CPU] [ASID_LENGTH];


void 		init_tlb (void);
void 		init_tlb_cpu (void);
void 		wrapped_tlbrefill (context_t *registers);
void 		tlb_clear (void);
void 		tlb_clear_local (void);
void 		tlb_clear_asid (const uint8_t asid);
void 		tlb_clear_local_asid (const uint8_t asid);
void 		tlb_clear_range (const uint32_t low_address, const uint32_t high_address, uint8_t asid);
void 		tlb_clear_local_range (const uint32_t low_address, const uint32_t high_address, uint8_t asid);
uint32_t 	get_virtual_space (thread_t thread);

#endif
