/**
 * @file utils.h
 *
 * Virtual memory management utils.
 *
 */

#ifndef UTILS_H_
#define UTILS_H_


#include <include/shared.h>
#include <include/c.h>
#include <drivers/printer.h>
#include <exc/assert.h>
#include <proc/thread.h>
#include <mm/vmm.h>

/* 4kB pages
 *
 * page 
 *   - 12 bits are zero
 *   - 20 bits are page bits
 * 
 * offset
 *   - 20 bits are zero
 *   - 12 bits are offset bits
 *	
 */   

#define UTILS_H_PAGE_SIZE 20
#define UTILS_H_PAGE_MASK 0xFFFFF000
#define UTILS_H_PAGE_CHECK 0xFFF00000
#define UTILS_H_OFFSET_SIZE 12
#define UTILS_H_OFFSET_MASK 0x00000FFF
#define UTILS_H_OFFSET_CHECK UTILS_H_PAGE_MASK
#define UTILS_H_ENTRYLO_01_MASK 0x03FFFFFF
#define UTILS_H_PP_SHIFT 6
#define UTILS_H_VP_MUL_SHIFT 13
#define UTILS_H_ASID_MASK 0x000000FF

uint32_t page (uint32_t address);

uint32_t offset (uint32_t address);

uint32_t address (uint32_t page, uint32_t offset);

uint32_t even_page (uint32_t page);

uint32_t odd_page (uint32_t page);

//uint32_t physical_page (vm_seci_t** virtual_map, uint32_t virtual_page);

uint32_t next_page (uint32_t page);

uint32_t is_page_even (uint32_t page);

uint32_t is_page_odd (uint32_t page);

uint32_t is_next_page (uint32_t offset);

uint32_t get_asid (void);

void tlb_map ( 
	uint32_t virtual_page,
	uint32_t physical_page0,
	uint32_t physical_page1,
	uint32_t asid,
	uint32_t flags_page0,
	uint32_t flags_page1,
	int32_t index);

int copy_from_thread (
	const thread_t thr,
	void* dest,
	const void* src,
	const size_t len);

int copy_to_thread (
	const thread_t thr,
	void* dest,
	const void* src,
	const size_t len);

#endif

