/**
 * @file virtmem.h
 *
 * Help methods for virtual memory manager.
 * These methods are called in the general functions of virtual memory manager. The methods are not synchronized, 
 * the synchronization must be provided by calling method. Every particularly searching in VM space have linear 
 * complexity for VM_MAX_AREAS.
 * @link kernel/mm/vmm.h Virtual memory manager @endlink
 *
 * @see vma_map
 * @see vma_unmap
 */

#ifndef VIRTMEM_H_
#define VIRTMEM_H_

#include <include/c.h>
#include <mm/physmem.h>
#include <proc/thread.h>
#include <mm/vmm.h>

/* Commented with implementation. */

void	vm_create_mapping	(thread_t thread, uint32_t virtual_address, uint32_t physical_address, size_t size);
void	vm_add_area		(thread_t thread, uint32_t from, size_t size);
void	set_searching_addresses	(uint32_t* ssaddress, uint32_t* seaddress, uint32_t from, const size_t size, const vm_flags_t flags);
int	search_avail_block	(vm_area_t** areas, uint32_t* from, const size_t size, uint32_t ss_address, uint32_t se_address);
int	check_access		(const uint32_t start_address, const size_t size, uint32_t* ss_address, uint32_t* se_address, const vm_flags_t flags);
int	add_mapping		(thread_t thread, uint32_t address, const size_t size, const vm_flags_t flags);
bool	vm_block_is_unmapped	(thread_t thread, uint32_t address, size_t size);
int 	vma_unmap_block 	(thread_t thread, const int index);

#endif

