/**
 * @file cvar.h
 *
 * Conditional variables are made of enhanced semaphores.
 *
 */


#ifndef CVAR_H_
#define CVAR_H_


#include <include/shared.h>
#include <include/c.h>
#include <adt/list.h>
#include <sync/spinlock.h>
#include <proc/thread.h>
#include <sync/semaphore.h>
#include <sync/mutex.h>


/** structure for semaphore */
struct cond
{
	/* condition variables are implemented by simple semaphore with enhanced functions */
	struct semaphore internal_semaphore;
};


/* initialization of cond structure */
void cond_init (struct cond * cvar);


/* freeing of cond structure */
void cond_destroy (struct cond * cvar);


/* set one thread free */
void cond_signal (struct cond * cvar);


/* set all threads free */
void cond_broadcast (struct cond * cvar);


/* thread will wait on condition variable for freeing */
void cond_wait (struct cond * cvar, struct mutex * mtx);


/* thread will wait on condition variable for freeing or time limit */
int cond_wait_timeout (struct cond * cvar, struct mutex * mtx, const unsigned int usec);


#endif