/**
 * @file mutex.h
 *
 * Mutes is implemented by semaphore with enhanced functions.
 *
 */


#ifndef MUTEX_H_
#define MUTEX_H_


#include <adt/list.h>
#include <sync/semaphore.h>
#include <proc/thread.h>


/** structure for mutex */
struct mutex
{
	/* mutex is implemented by semaphore */
	struct semaphore sem;
	/* owning thread */
	thread_t owner;
};

/* initialization of structure for user synchronization */
void init_u_sync (void);

/* initialization of mutex structure (u - user) */
void mutex_init (struct mutex * mtx);
int u_mutex_init (void** mtx);

/* freeing of mutex structure (u - user) */
void mutex_destroy (struct mutex * mtx);
int u_mutex_destroy (void* mtx);

/* lock down this mutex (u - user) */
void mutex_lock (struct mutex * mtx);
int u_mutex_lock (void* mtx);

/* lock down this mutex for some max time (u - user) */
int mutex_lock_timeout (struct mutex * mtx, const unsigned int usec);
int u_mutex_lock_timeout (void* mtx, const unsigned int usec);

/* lock up this mutex (u - user) */
int u_mutex_unlock (void* mtx);

/** mutex unlock up
 *
 * @param mtx pointer to mutex structure
 *
 */
static inline void mutex_unlock (struct mutex * mtx)
{
#ifdef DEBUG_MUTEX
	/* control of unlocking by another thread */
	if ((0 < DEBUG_MUTEX) && (thread_get_current() != mtx->sem.last_owner))
	{
		/* panic in unlocking by another thread */
		panic ("mutex unlocked by bad guy");
	}
#endif
	/* raising one count semaphore is mutex unlock */
	sem_up_limited (&(mtx->sem), 1);
}


#endif