/**
 * @file rwlock.h
 *
 * Read-write lock is based on read-lock semaphore and write-lock mutex.
 *
 */


#ifndef RWLOCK_H_
#define RWLOCK_H_


#include <sync/semaphore.h>
#include <sync/mutex.h>


/**  structure for rwlock */
struct rwlock
{
	/* write lock is realized by simple mutex */
	struct mutex write_lock;
	/* read lock is realized by semaphore */
	struct semaphore read_lock;
};


/* initialize rwlock data structure */
void rwlock_init (struct rwlock * rwl);


/* clean up and free rwlock data structure */
void rwlock_destroy (struct rwlock * rwl);


/* locks target lock for reading */
void rwlock_read_lock (struct rwlock * rwl);


/* locks target lock for writting */
void rwlock_write_lock (struct rwlock * rwl);


/* locks target lock for reading until time limit expires */
int rwlock_read_lock_timeout (struct rwlock * rwl, const unsigned int usec);


/* locks target lock for writting until time limit expires */
int rwlock_write_lock_timeout (struct rwlock * rwl, const unsigned int usec);


/* unlocks target lock for writting */
void rwlock_write_unlock (struct rwlock * rwl);


/* unlocks target lock for reading */
void rwlock_read_unlock (struct rwlock * rwl);


#endif