/**
 * Global synchronized linked list test #2
 */

static char * desc =
    "Global synchronized linked list test #2\n"
    "Spawns a number of threads and makes them all insert the one\n"
    "item to the list. Only one thread can insert it. Test\n"
    "of insert and find operation. \n\n";

#include <api.h>
#include "../include/defs.h"


/*
 * The number of threads of each kind to run concurrently.
 */
#define THREAD_COUNT 100

/*
 * The number of remove cycles for every thread.
 */
#define INSERT_CYCLES 100

/*
 * The concurrent list.
 */
static sync_list_t list;

/*
 * The read-write lock.
 * First we need to create all the threads, then unlock the section.
 */
static struct rwlock rwlock;

/*
 * Array of records, which thread insert the item.
 */
static volatile bool inserted[THREAD_COUNT];

/*
 * Was the item inserted to the list?
 */
static volatile bool item_inserted = false;

/*
 * The function that repeatly tries to insert the one item to the global synchronized 
 * linked list.
 *
 * @param data The ID of the thread.
 */
static void *
thread_item_inserting (void * data)
{
	int thread_id = (int) data;
	void * list_data;
	bool result;
	inserted[thread_id] = false;
	
	rwlock_read_lock (&rwlock);
	
	for (int i=0; i<INSERT_CYCLES; ++i)
	{
		result = sync_list_insert (&list, (unsigned long)KEY_MAGIC, DATA_MAGIC);
		if (result)
		{
			inserted[thread_id] = true;
			item_inserted = true;
		}
		
		if(item_inserted)
		{
			assert(sync_list_find (&list, (unsigned long)KEY_MAGIC, &list_data));
			assert(list_data == DATA_MAGIC);
		}
	}
	
	rwlock_read_unlock (&rwlock);
	
	return NULL;
}


void
run_test (void)
{
	
	rwlock_init (&rwlock);
	
	/* Init. */
	sync_list_init (&list);
	
	int cnt;
	thread_t threads [THREAD_COUNT];
	
	printk (desc);
	
	rwlock_write_lock (&rwlock);
	
	for (cnt= 0; cnt < THREAD_COUNT; ++cnt) 
	{
		threads [cnt] = robust_thread_create (
		    thread_item_inserting, (void*)cnt);
	}
	
	rwlock_write_unlock (&rwlock);
	
	/*
	 * Wait for all the threads to finish.
	 */
	printk ("Waiting for all the threads to finish...\n");
	
	/*
	 * Reap the threads.
	 */
	for (cnt = 0; cnt < THREAD_COUNT; ++cnt) 
	{
		robust_thread_join (threads [cnt]);
	}
	
	rwlock_destroy (&rwlock);
	
	printk ("Items in the list: (should be the one inserted item)\n");
	sync_list_dump (&list);
	
	/* How many threads removed the item. */
	int counter = 0;
	for (int i=0; i<THREAD_COUNT; ++i)
	{
		if(inserted[i])
		{
			++counter;
		}
	}
	
	if (counter == 1) {
		printk ("Test passed...\n");
	} else {
		printk ("Test failed...\n");
	}
}
